/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;
using System.ComponentModel;
using xGameConsoleXNA.Properties;

namespace xGameConsoleXNA
{
    namespace GenericCommands
    {

        public class CommandLogger : BaseCommands
        {
            private bool mblnLogging = false;
            private string mstrFile = string.Empty;


            public CommandLogger(xGameConsoleXNA.Console Console)
                : base(Console)
            {
            }

            /// <summary>
            /// Overides the CommandEvents_CommandExecuted in the base object
            /// </summary>
            /// <remarks>This method will be called every time a command is executed in the console.</remarks>
            protected override void CommandEvents_CommandExecuted(object Sender, ReturnValueEventArgs e)
            {
                // check if a filename has been set or logging is not enabled
                if (mstrFile == string.Empty || !mblnLogging) return;

                string Tmp = string.Empty;
                string Cmd = e.Command;

                // try to get the arguments
                try
                {
                    Tmp = string.Join(" ", e.Args);
                }
                catch { }

                // save the command and arguments out to the logging file
                System.IO.StreamWriter F = null;
                try
                {
                    F = new System.IO.StreamWriter(mstrFile, true);
                    F.WriteLine(e.Command + " " + Tmp);
                }
                catch { }
                finally
                {
                    if (F != null)
                    {
                        F.Flush();
                        F.Close();
                    }
                    F = null;
                }
            }

            [Category("Logging"), Description("Sets the file that the recorded commands will be saved to.")]
            internal string SetLoggingFile(string[] Args, bool usedAsFunction)
            {

                StringCollection output = new StringCollection();
                string Tmp = string.Empty;
                try
                {
                    Tmp = string.Join(" ", Args);
                }
                catch { }

                // check if no file was specified
                if (Tmp == string.Empty)
                {
                    output.Add(Resources.NoFileSpecified);
                }
                else
                {
                    // check if there is a directory
                    string Dir = System.IO.Path.GetDirectoryName(Tmp);
                    if (Dir != string.Empty)
                    {
                        // check for invalid path characters
                        if (Dir.IndexOfAny(System.IO.Path.GetInvalidPathChars()) != -1)
                        {
                            output.Add(Resources.InvalidCharacters);
                        }
                        else
                        {
                            // check if the directory exists
                            if (System.IO.Directory.Exists(Dir) == false)
                            {
                                output.Add(Resources.DirectoryNameNonExistant + " " + Dir);
                            }
                        }
                    }
                }

                // check for invalid character
                if (System.IO.Path.GetFileName(Tmp).IndexOfAny(System.IO.Path.GetInvalidFileNameChars()) != -1)
                {
                    output.Add(Resources.InvalidCharacters);
                }                     

                // if no output yet it means every thing went well
                if (output.Count == 0)
                {
                    output.Add(Resources.LoggingFileSetTo.Replace(Resources.RKEY, Tmp));
                    // set the logging file
                    mstrFile = Tmp;
                }

                if (!usedAsFunction) this.Print(output);
                // if used as a function return logging file
                return usedAsFunction ? mstrFile : string.Empty;
            }

            [Category("Logging"), Description("Begins recording commands typed into the console. \nUse /d or /delete to delete any existing log file and " +
                                              "\nwill return False if existing log file could not be deleted.")]
            internal string StartLoggingCommands(string[] Args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();

                // check if a logging file has been specified
                if (mstrFile == string.Empty)
                {
                    output.Add(Resources.NoFileSpecified);
                }
                else
                {
                    // check if at least one argument was specified
                    if (Args != null && Args.Length > 0)
                    {
                        string tmp = string.Empty;
                        tmp = string.Join(" ", Args).ToLower();
                        // check if the user wants to delete the existing log file
                        if ((tmp == "/d" || tmp == "/delete") && System.IO.File.Exists(mstrFile))
                        {
                            // try to delete the log file
                            try
                            {
                                System.IO.File.Delete(mstrFile);
                            }
                            catch
                            {
                                output.Add(Resources.CouldNotDeleteFile);
                            }
                        }
                    }
                }

                // set logging to true of no errors occoured
                if (output.Count == 0)
                {
                    mblnLogging = true;
                }

                if (!usedAsFunction) this.Print(output);
                // if used as a function return logging status
                return usedAsFunction ? mblnLogging.ToString() : string.Empty;
            }

            [Category("Logging"), Description("Stops recording commands typed into the console.")]
            internal string StopLoggingCommands(string[] Args, bool usedAsFunction)
            {
                // set logging to false
                mblnLogging = false;

                if (!usedAsFunction) this.Print(Resources.CommandloggingHasStopped);
                // if used as a function return logging status
                return usedAsFunction ? mblnLogging.ToString() : string.Empty;
            }

            [Category("Logging"), Description("Returns the name of the logging file.")]
            internal string GetLoggingFile(string[] Args, bool usedAsFunction)
            {
                if (!usedAsFunction) this.Print(mstrFile);
                // if used as a function return logging file
                return usedAsFunction ? mstrFile : string.Empty;
            }

            [Category("Logging"), Description("Returns true if commands are being recorded.")]
            internal string IsLoggingCommands(string[] Args, bool usedAsFunction)
            {
                if (!usedAsFunction) this.Print(mblnLogging.ToString());
                // if used as a function return logging status
                return usedAsFunction ? mblnLogging.ToString() : string.Empty;
            }

            public override void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
            {
                base.OnConnect(Manager, ExtenderManager);

                DoAddCommand(Resources.CMD_setloggingfile, new xGameConsoleXNA.CommandCallback(SetLoggingFile));
                DoAddCommand(Resources.CMD_getloggingfile, new xGameConsoleXNA.CommandCallback(GetLoggingFile));
                DoAddCommand(Resources.CMD_startloggingcommands, new xGameConsoleXNA.CommandCallback(StartLoggingCommands));
                DoAddCommand(Resources.CMD_stoploggingcommands, new xGameConsoleXNA.CommandCallback(StopLoggingCommands));
                DoAddCommand(Resources.CMD_isloggingcommands, new xGameConsoleXNA.CommandCallback(IsLoggingCommands));
            }
        }
    }
}
