/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;
using System.ComponentModel;
using xGameConsoleXNA.Properties;


namespace xGameConsoleXNA
{
    namespace GenericCommands
    {

        public class FileCommands : BaseCommands
        {

            public FileCommands(xGameConsoleXNA.Console Console)
                : base(Console)
            {
            }

            [Category("File"), Description("Displays the current directory.")]
            public string GetCurrentDirectory(string[] args, bool usedAsFunction)
            {
                if (!usedAsFunction) this.Print(System.IO.Directory.GetCurrentDirectory());
                return System.IO.Directory.GetCurrentDirectory();
            }

            [Category("File"), Description("Displays a list of files in the current directory.")]
            public string ListFiles(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();


                output.Add("List of files found");
                output.Add(new string('-', output[0].Length));
                output.AddRange(System.IO.Directory.GetFiles("."));
                int Count = output.Count - 2;
                output.Add(Count.ToString() + " file(s)");


                if (!usedAsFunction) this.Print(output);
                // remove header and footers
                output.RemoveAt(0);
                output.RemoveAt(0);
                output.RemoveAt(output.Count - 1);
                // return value
                return usedAsFunction ? Helpers.StringCollectionToString(output, "\n") : string.Empty;
            }

            [Category("File"), Description("Sets the current directory.")]
            public string SetCurrentDirectory(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string path = string.Empty;
                string curDir = System.IO.Directory.GetCurrentDirectory();
                // get the specified path
                try
                {
                    path = string.Join(string.Empty, args);
                }
                catch { }

                // try to set path
                try
                {
                    System.IO.Directory.SetCurrentDirectory(path);
                    curDir = System.IO.Directory.GetCurrentDirectory();
                    output.Add("Current directory is " + curDir);
                }
                catch (Exception)
                {
                    output.Add("Invalid path was specified.");
                }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? curDir : string.Empty;
            }

            public override void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
            {
                base.OnConnect(Manager, ExtenderManager);
                DoAddCommand(Resources.CMD_dir, new xGameConsoleXNA.CommandCallback(this.ListFiles));
                DoAddCommand(Resources.CMD_curdir, new xGameConsoleXNA.CommandCallback(this.GetCurrentDirectory));
                DoAddCommand(Resources.CMD_cd, new xGameConsoleXNA.CommandCallback(this.SetCurrentDirectory));
            }
        }
    }

}
