/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;
using System.ComponentModel;
using xGameConsoleXNA.Properties;

namespace xGameConsoleXNA
{
    namespace GenericCommands
    {

        public class GenericCommands : BaseCommands
        {


            #region " Commands ... "
            [Category("Generic"), Description("Displays a list of command categories or displays a list of commands for a specified category.\n" +
                                              "Example: cat >category<")]
            private string ShowCategory(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();

                // check if there are any arguments and if there are none just list off all of the categories.
                if (args == null || args.Length == 0)
                {
                    // get a list of categories and add it to the items list
                    string[] cats;
                    cats = this.CommandManager.GetCategoryList();
                    if (cats != null)
                    {
                        output.Add(Resources.Categories);
                        output.Add(new string('-', 12));
                        output.AddRange(cats);
                        output.Add(Resources.NumberCategories.Replace(Resources.RKEY, cats.Length.ToString()));
                    }
                }
                // check if there was specified arguments
                else if (args != null && args.Length >= 1)
                {
                    // get category name
                    string catName = string.Join(" ", args);
                    if (catName.Trim().ToLower() != string.Empty)
                    {
                        // get a list of cammond that belong to the specified category
                        string[] cmds;
                        cmds = this.CommandManager.GetCommandList(catName.Trim());

                        // check if any commands were returned
                        if (cmds != null)
                        {
                            // add the commands to the items list
                            output.Add(Resources.Commands);
                            output.Add(new string('-', 12));
                            output.AddRange(cmds);
                            output.Add(Resources.NumberCommands.Replace(Resources.RKEY, cmds.Length.ToString()));
                        }
                        else
                        {
                            // report that there were no commands to be found
                            output.Add(Resources.NoCommandsFound);
                        }
                    }
                }

                // write text to console
                if (!usedAsFunction) this.Print(output);
                // remove first 3 lines and the last line
                output.RemoveAt(output.Count - 1);
                if (output.Count >= 3)
                {
                    //  remove the first 2 description items
                    output.RemoveAt(0);
                    output.RemoveAt(0);
                }
                // return command list if used a function
                return usedAsFunction ? Helpers.StringCollectionToString(output, "\n") : string.Empty;
            }

            [Category("Generic"), Description("Loads a set of command aliases from a file.")]
            public string LoadAliasList(string[] args, bool usedAsFunction)
            {
                // get filename specified
                string File = string.Empty;
                try
                {
                    File = string.Join(" ", args);
                }
                catch { }

                // get alias names
                string[] lines = null;
                System.IO.StreamReader reader = null;
                try
                {
                    // read ll data from the file
                    reader = new System.IO.StreamReader(File);
                    lines = reader.ReadToEnd().Split(new string[] { "\n" }, StringSplitOptions.RemoveEmptyEntries);
                    // ReadAllLines method not supported on 360
                    //lines = System.IO.File.ReadAllLines(File);   
                }
                catch { }
                finally
                {
                    if (reader != null)
                    {
                        reader.Close();
                        reader.Dispose();
                    }
                    reader = null;
                }

                StringCollection output = new StringCollection();

                // check if any lines were read
                int Count = 0;
                if (lines != null)
                {
                    foreach (string item in lines)
                    {
                        string Comm;
                        string aName;
                        int Index;
                        Comm = item;

                        // check for alias / command separation otherwise ignore line
                        Index = Comm.IndexOf(" ");
                        if (Index != -1)
                        {
                            // get alias name and command
                            aName = Comm.Substring(0, Index).Trim();
                            Comm = Comm.Substring(Index).Trim();
                            try
                            {
                                // add alias
                                mobjManager.AddAlias(aName, Comm);
                                Count++;
                            }
                            catch (Exception arge)
                            {
                                // write errormsg to console
                                if (!usedAsFunction) output.Add(arge.Message);
                            }
                        }
                    }
                }

                // write to console how many aliases were loaded
                output.Add(Resources.NumberAliasLoaded.Replace(Resources.RKEY, Count.ToString()));

                if (!usedAsFunction) this.Print(output);
                // if used as a function return number of items loaded
                return usedAsFunction ? Count.ToString() : string.Empty;
            }

            [Category("Generic"), Description("Saves the current set of command aliases to a file.")]
            public string SaveAliasList(string[] args, bool usedAsFunction)
            {
                string File = string.Empty;
                try
                {
                    File = string.Join(" ", args);
                }
                catch { }

                // get alias names
                string[] Names;
                bool saved = false;
                Names = mobjManager.GetAliasList();
                System.IO.FileStream writer = null;
                try
                {
                    if (System.IO.File.Exists(File)) System.IO.File.Delete(File);
                }
                catch { }

                // try to save the data to disk
                try
                {
                    writer = new System.IO.FileStream(File, System.IO.FileMode.Create, System.IO.FileAccess.Write);
                    foreach (string item in Names)
                    {
                        string Comm;
                        Comm = mobjManager.GetCommandFromAlias(item);
                        string output = item + " " + Comm;
                        if (Comm != null && output.Length > 0)
                        {
                            byte[] bytes = new byte[output.Length];
                            for (int idx = 0; idx < bytes.Length; idx++)
                            {
                                bytes[idx] = (byte)(output[idx]);
                            }
                            writer.Write(bytes, 0, bytes.Length);
                        }
                    }
                    saved = true;
                }
                catch { }
                finally
                {
                    if (writer != null)
                    {
                        writer.Flush();
                        writer.Close();
                    }
                    writer = null;
                }

                return usedAsFunction && saved ? true.ToString() : false.ToString();
            }

            [Category("Generic"), Description("Displays help for a command. \nExample: help >command or command alias<")]
            private string ShowHelp(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                System.ComponentModel.DescriptionAttribute desc = null;

                // check if command or alias name specified
                if (args == null || args.Length == 0)
                {
                    output.Add(Resources.NoHelpArgument);
                }
                else
                {
                    // try to retrieve specified commmand
                    CommandCallback item;
                    item = mobjManager.GetCommand(args[0].Trim(), true);
                    if (item == null)
                    {
                        output.Add(Resources.SpecifiedCommandNotExist);
                    }
                    else
                    {

                        // try to get description sttribute
                        desc = Helpers.GetAttribute<System.ComponentModel.DescriptionAttribute>(item.Method);

                        // write description out to console if present
                        if (desc != null)
                        {
                            output.Add(desc.Description);
                        }
                        else
                        {
                            output.Add(Resources.NoHelpAvailible);
                        }
                    }
                }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction && desc != null ? desc.Description : string.Empty;
            }

            [Category("Generic"), Description("Displays a list of commands registered with the console.")]
            private string ListCommands(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string[] Commands;
                Commands = mobjManager.GetCommandList();
                output.Add(Resources.Commands);
                output.Add(new string('-', 12));
                output.AddRange(Commands);
                output.Add(Resources.NumberCommands.Replace(Resources.RKEY, Commands.Length.ToString()));

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? Helpers.StringArrayToString(Commands, 0, "\n") : string.Empty;
            }


            //protected override void CommandEvents_CommandExecuted(object Sender, ReturnValueEventArgs e)
            //{
            //    if (e.CommandString.StartsWith(Resources.CMD_echo))
            //    {
            //        this.Print(e.ArgumentString);
            //    }
            //}

            [Category("Generic"), Description("Writes text out to the console.")]
            private string Echo(string[] args, bool usedAsFunction)
            {
                string ret = string.Empty;
                try
                {
                    ret = string.Join(string.Empty, args);
                }
                catch { }

                if (!usedAsFunction) this.Print(ret);
                return usedAsFunction ? ret : string.Empty;
            }

            [Category("Generic"), Description("Deletes an alias for a command.")]
            private string DeleteAlias(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string aliasName = string.Empty;

                if (args != null && args.Length > 0)
                {
                    try
                    {
                        // try to remove the alias
                        mobjManager.RemoveAlias(args[0]);
                        aliasName = args[0].Trim();
                        output.Add(Resources.AliasRemoved);
                    }
                    catch (Exception ex)
                    {
                        output.Add(ex.Message);
                    }
                }
                else
                {
                    output.Add(Resources.NoAliasWasSpecified);
                }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? aliasName : string.Empty;
            }

            [Category("Generic"), Description("Creates an alias for a command. \nExample: createalias >alias name< >command to execute<")]
            private string CreateAlias(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string aliasName = string.Empty;

                if (args != null && args.Length >= 2)
                {
                    // get the specified command
                    string Cmd = string.Empty;
                    Cmd = string.Join(string.Empty, args, 1, args.Length - 1);
                    try
                    {
                        // attempt to create the alias
                        mobjManager.AddAlias(args[0], Cmd);
                        aliasName = args[0].Trim();
                        output.Add(Resources.AliasCreated + " " + Resources.Alias + " \'" + args[0].Trim() + "\' " + Resources.executes + " >" + Cmd);
                    }
                    catch (Exception arg)
                    {
                        output.Add(arg.Message);
                    }
                }
                else
                {
                    output.Add(Resources.AliasWasNotCreated);
                    output.Add(Resources.CreateAliasExample);
                }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? aliasName : string.Empty;
            }

            [Category("Generic"), Description("Displays the current list of command aliases.")]
            private string ListAlias(string[] args, bool usedAsFunction)
            {

                StringCollection output = new StringCollection();
                output.Add(string.Empty);
                output.Add(Resources.Alias + "                    " + Resources.Command);
                output.Add(new string('-', output[output.Count - 1].Length));

                // get list of aliases
                string[] Items;
                Items = mobjManager.GetAliasList();
                if (Items != null && Items.Length > 0)
                {
                    foreach (string item in Items)
                    {
                        output.Add(item + new string(' ', Math.Max(25, item.Length) - Math.Min(25, item.Length)) + mobjManager.GetCommandFromAlias(item));
                    }
                }

                output.Add("");
                output.Add(mobjManager.AliasCount.ToString() + (mobjManager.AliasCount == 1 ? " alias." : " aliases."));

                if (!usedAsFunction) this.Print(output);

                // remove first 3 lines
                output.RemoveAt(0);
                output.RemoveAt(0);
                output.RemoveAt(0);
                // remove last line
                output.RemoveAt(output.Count - 1);

                // return string that was printed to console
                return usedAsFunction ? Helpers.StringCollectionToString(output, "\n") : string.Empty;
            }

            /*  SetConsoleSize and SetConsoleSpeed are commented out because they would not berelevant if the user implemented there
             *  own rendering system for displayig the console.
            [Category("Generic"), Description("Specifies the size of the console in percentages it takes up on screen.")]
            private string SetConsoleSize(string[] args, bool usedAsFunction)
            {
                if (mblnIsConnected && (mobjConsoleReference != null))
                {

                    try
                    {
                        if (args == null)
                        {
                            this.Print("MaxConsolePercentage: " + mobjConsoleReference.MaxConsolePercentage.ToString());
                        }
                        else
                        {
                            float NewSize = float.Parse(args[0]);

                            if (NewSize > 100 || NewSize < 5.0F)
                            {
                                this.Print("consolesize must be between 5 and 100");
                            }
                            else
                            {

                                mobjConsoleReference.MaxConsolePercentage = NewSize;
                                if (mobjConsoleReference.CurrentConsolePercentage < mobjConsoleReference.MaxConsolePercentage && mobjConsoleReference.IsVisible)
                                {
                                    mobjConsoleReference.State = Console.ConsoleState.Opening;
                                }
                            }
                        }
                    }
                    catch
                    {
                        //Console.Instance.
                        this.Print("consolesize must be between 5 and 100");
                    }
                }

                return null;
            }

            [Category("Generic"), Description("Specifies the speed at which the console opens and closes.")]
            private string SetConsoleSpeed(string[] args, bool usedAsFunction)
            {
                if (mblnIsConnected && (mobjConsoleReference != null))
                {

                    try
                    {
                        if (args == null)
                        {
                            this.Print("Speed: " + mobjConsoleReference.ConsoleSpeed.ToString());
                        }
                        else
                        {
                            float Value = float.Parse(args[0]);

                            if (Value < 0.0F)
                            {
                                this.Print("Speed must be greater then 0.0");
                            }
                            else
                            {
                                mobjConsoleReference.ConsoleSpeed = Value;
                            }
                        }
                    }
                    catch
                    {
                        //Console.Instance.
                        // mobjExtenderManager.Console.Print("consolesize must be between 5 and 100")
                    }
                }

                return null;
            }        */

            [Category("Generic"), Description("Clears the console.")]
            private string Clear(string[] args, bool usedAsFunction)
            {
                // clear the console history
                mobjConsoleReference.History.Clear();
                // return empty string
                return string.Empty;
            }
            #endregion

            public override void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
            {
                base.OnConnect(Manager, ExtenderManager);
                if (mobjManager == null)
                {
                    return;
                }
                DoAddCommand(Resources.CMD_commands, new xGameConsoleXNA.CommandCallback(this.ListCommands));
                DoAddCommand(Resources.CMD_echo, new xGameConsoleXNA.CommandCallback(this.Echo));
                DoAddCommand(Resources.CMD_createalias, new xGameConsoleXNA.CommandCallback(this.CreateAlias));
                DoAddCommand(Resources.CMD_deletealias, new xGameConsoleXNA.CommandCallback(this.DeleteAlias));
                DoAddCommand(Resources.CMD_alias, new xGameConsoleXNA.CommandCallback(this.ListAlias));
                DoAddCommand(Resources.CMD_help, new xGameConsoleXNA.CommandCallback(this.ShowHelp));
                DoAddCommand(Resources.CMD_category, new xGameConsoleXNA.CommandCallback(this.ShowCategory));
                DoAddCommand(Resources.CMD_clear, new xGameConsoleXNA.CommandCallback(this.Clear));
                DoAddCommand(Resources.CMD_savealias, new xGameConsoleXNA.CommandCallback(SaveAliasList));
                DoAddCommand(Resources.CMD_loadalias, new xGameConsoleXNA.CommandCallback(LoadAliasList));
                //  DoAddCommand("consolesize", new xGameConsoleXNA.CommandCallback(this.SetConsoleSize));
                //  DoAddCommand("consolespeed", new xGameConsoleXNA.CommandCallback(this.SetConsoleSpeed));
            }

            public GenericCommands(xGameConsoleXNA.Console Console)
                : base(Console)
            {
            }
        }
    }

}
