/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;
using System.ComponentModel;
using xGameConsoleXNA.Properties;

namespace xGameConsoleXNA
{
    namespace GenericCommands
    {

        public class LogicCommands : BaseCommands
        {
            private string DoIfCommand(string[] Args, bool usedAsFunction, bool compareValue)
            {
                StringCollection output = new StringCollection();
                string returnValue = string.Empty;
                // try to check if the first argument is true
                if (Args != null && Args.Length > 1 && Args[0] != null)
                {
                    // try to eveluate the first argument
                    bool val = false;
                    string expr = Args[0].Trim();
                    if (expr == bool.FalseString || expr == bool.FalseString.ToLower() || expr == bool.FalseString.ToUpper() ||
                        expr == string.Empty || expr[0] == '0')
                    {
                        val = false;
                    }
                    else
                    {
                        val = true;
                    }


                    // if val is same as comparevalue then we can try and run the command
                    if (val == compareValue)
                    {
                        string tmp = string.Empty;

                        try
                        {
                            tmp = string.Join(string.Empty, Args, 1, Args.Length - 1);
                        }
                        catch { }

                        // call the specified command if present and get it's return value
                        returnValue = mobjManager.Execute(tmp, usedAsFunction);
                       // output.Add(Resources.Returned + " '" + returnValue + "'");
                    }
                }
                else
                {
                    output.Add(Resources.MissingOrInvalidArg);
                }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? returnValue : string.Empty;
            }

            #region " Commands ... "
            [Category("Logic"), Description("If the first argument evaluates to true this command will execute the string specified in the second argument, otherwise the third argument will execute.")]
            private string IfCommand(string[] Args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string returnValue = string.Empty;
                // try to check if the first argument is true
                if (Args != null && Args.Length > 0 && Args[0] != null)
                {
                    // try to eveluate the first argument
                    bool val = false;
                    string expr = Args[0].Trim();

                    if (expr == bool.FalseString || expr == bool.FalseString.ToLower() || expr == bool.FalseString.ToUpper() ||
                        expr == string.Empty || expr[0] == '0')
                    {
                        val = false;
                    }
                    else 
                    {
                        val = true;
                    }


                    // if val is same as comparevalue then we can try and run the command
                    if (val && Args.Length > 1)
                    {                    
                        // call the specified command in argument 1 and get it's return value
                        returnValue = mobjManager.Execute(Args[1], usedAsFunction);
                       // output.Add(Resources.Returned + " '" + returnValue + "'");
                    }
                    else if ( Args.Length > 2)
                    {
                        // call the specified command in argument 1 and get it's return value
                        returnValue = mobjManager.Execute(Args[2], usedAsFunction);
                       // output.Add(Resources.Returned + " '" + returnValue + "'");
                    }
                }
                else
                {
                    output.Add(Resources.MissingOrInvalidArg);
                }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? returnValue : string.Empty;
            }
            
            [Category("Logic"), Description("If the first argument evaluates to true this command will execute the specified command.")]
            private string IfTrueCommand(string[] Args, bool usedAsFunction)
            {
                return this.DoIfCommand(Args, usedAsFunction, true);
            }

            [Category("Logic"), Description("If the first argument evaluates to false this command will execute the specified command.")]
            private string IfFalseCommand(string[] Args, bool usedAsFunction)
            {
                return this.DoIfCommand(Args, usedAsFunction, false);
            }

            [Category("Logic"), Description("Compars two values and returns true or false.")]
            private string EqualsCommand(string[] Args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string returnValue = string.Empty;
                if (Args == null || Args.Length != 2)
                {
                    output.Add(Resources.Expected2Args);
                }
                else
                {
                    // perform comparison  	
                    returnValue = (Args[0] == Args[1].TrimStart() ? bool.TrueString : bool.FalseString);
                    output.Add(returnValue);
                }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? returnValue : string.Empty;
            }

            #endregion

            public LogicCommands(xGameConsoleXNA.Console Console)
                : base(Console)
            {
            }

            public override void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
            {
                base.OnConnect(Manager, ExtenderManager);
                DoAddCommand(Resources.CMD_iftrue, new xGameConsoleXNA.CommandCallback(this.IfTrueCommand));
                DoAddCommand(Resources.CMD_iffalse, new xGameConsoleXNA.CommandCallback(this.IfFalseCommand));
                DoAddCommand(Resources.CMD_equals, new xGameConsoleXNA.CommandCallback(this.EqualsCommand));
                DoAddCommand(Resources.CMD_if, new xGameConsoleXNA.CommandCallback(this.IfCommand));
            }
        }
    }
}
