/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;
using System.ComponentModel;
using xGameConsoleXNA.Properties;


namespace xGameConsoleXNA
{
    namespace GenericCommands
    {
        public class SimpleScripting : BaseCommands
        {
            public class Script : IDisposable
            {
                protected internal StringCollection mobjLines = new StringCollection();
                protected internal int mintCurrentLine = -1;
                protected internal Stack<int> mobjLoops = new Stack<int>();
                protected internal BackgroundWorker worker = new BackgroundWorker();

                public Script()
                {
                    this.worker.DoWork += new DoWorkEventHandler(worker_DoWork);
                }

                private void worker_DoWork(object sender, DoWorkEventArgs e)
                {
                    this.RunScript((CommandManager)e.Argument);
                }

                private void RunScript(CommandManager Manager)
                {
                 // begin running script 
                    while (true)
                    {
                        // move to next line
                        mintCurrentLine++;
                        // check if currnet line exceeds number of lines of script
                        if (mintCurrentLine > mobjLines.Count - 1)
                        {
                            mintCurrentLine = -1;
                            break;
                        }


                        // check if script if begining a loop
                        if (mobjLines[mintCurrentLine] == Resources.CMD_beginloop)
                        {
                            this.mobjLoops.Push(this.mintCurrentLine - 1);
                        }       // check if ending a loop
                        else if (mobjLines[mintCurrentLine] == Resources.CMD_endloop)
                        {
                            this.mintCurrentLine = this.mobjLoops.Pop();
                        }    // check if exiting a loop
                        else if (mobjLines[mintCurrentLine] == Resources.CMD_exitloop)
                        {
                            // begin search for an end loop
                            for (int idx = this.mintCurrentLine; idx <= this.mobjLines.Count - 1; idx++)
                            {
                                // check or end loop
                                if (this.mobjLines[idx].Trim() == Resources.CMD_endloop)
                                {
                                    // set current line ad exit
                                    this.mintCurrentLine = idx;
                                    break;
                                }
                            }
                        }
                        else  
                        {
                            // try to run line of script
                            Manager.Execute(mobjLines[mintCurrentLine]);
                        } 
                    } 
                }

                public virtual void Execute(CommandManager Manager)
                {
                    this.Execute(Manager, false);
                }

                public virtual void Execute(CommandManager Manager, bool async)
                {
                    if (async)
                    {
                        this.worker.RunWorkerAsync(Manager);
                    }
                    else
                    {
                        this.RunScript(Manager);
                    }
                }

                #region " IDisposable Support "
                private bool disposed = false;

                // IDisposable
                private void Dispose(bool disposing)
                {
                    if (!this.disposed)
                    {
                        if (disposing)
                        {
                            this.worker.DoWork -= new DoWorkEventHandler(worker_DoWork);
                            this.worker.Dispose();
                            this.worker = null;
                            this.mobjLines.Clear();
                            this.mobjLines = null;
                            this.mobjLoops.Clear();
                            this.mobjLoops = null;
                        }

                        // put code to free unmanaged resources here
                    }
                    this.disposed = true;
                }

                // This code added by Visual Basic to correctly implement the disposable pattern.
                public virtual void Dispose()
                {
                    // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
                    Dispose(true);
                    GC.SuppressFinalize(this);
                }

                ~Script()
                {
                    // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
                    Dispose(false);
                    //base.Finalize();
                }
                #endregion

            }

            // used to store each script
            protected internal Dictionary<string, Script> mobjScripts = new Dictionary<string, Script>();


            public SimpleScripting(xGameConsoleXNA.Console Console)
                : base(Console)
            {
            }

            public override void Dispose()
            {
                base.Dispose();
                foreach (string Scr in mobjScripts.Keys)
                {
                    mobjScripts[Scr].Dispose();
                }
                mobjScripts.Clear();
                mobjScripts = null;
            }

            #region " Commands ... "
           /* [Category("Scripting"), Description("Begins a loop."), VisibleToUsers(false)]
            public string BeginLoop(string[] args, bool usedAsFunction)
            {
                if (this.scriptStack.Count == 0) return string.Empty;

                Script script = this.scriptStack.Peek();
                // push another loop onto the stack
                script.mobjLoops.Push(script.mintCurrentLine - 1);

                return string.Empty;
            }

            [Category("Scripting"), Description("Definds the end of a loop."), VisibleToUsers(false)]
            public string EndLoop(string[] args, bool usedAsFunction)
            {
                if (this.scriptStack.Count == 0) return string.Empty;

                Script script = this.scriptStack.Peek();
                // restore current line to begining of loop
                script.mintCurrentLine = script.mobjLoops.Pop();

                return string.Empty;
            }

            [Category("Scripting"), Description("Exits a loop."), VisibleToUsers(false)]
            public string ExitLoop(string[] args, bool usedAsFunction)
            {
                if (this.scriptStack.Count == 0) return string.Empty;

                Script script = this.scriptStack.Peek();
                // begin search for an end loop
                for (int idx = script.mintCurrentLine; idx <= script.mobjLines.Count - 1; idx++)
                {
                    // check or end loop
                    if (script.mobjLines[idx].Trim() == Resources.CMD_endloop)
                    {
                        // set current line ad exit
                        script.mintCurrentLine = idx;
                        return string.Empty;
                    }
                }

                // set current line
                script.mintCurrentLine = script.mobjLines.Count;
                return string.Empty;
            } */

            [Category("Scripting"), Description("Lists the loaded scripts.")]
            private string ListScripts(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();

                output.Add(Resources.Scripts);
                output.Add(new string('-', output.Count));

                // add script names to list
                foreach (string Tmp in mobjScripts.Keys)
                {
                    output.Add(Tmp);
                }

                // report number of scripts
                output.Add(mobjScripts.Count.ToString() + " " + (mobjScripts.Count == 1 ? Resources.scriptDOT : Resources.scriptsDOT));

                if (!usedAsFunction) this.Print(output);
                output.RemoveAt(output.Count - 1);
                output.RemoveAt(0);
                output.RemoveAt(0);
                return usedAsFunction ? Helpers.StringCollectionToString(output, "\n") : string.Empty;
            }

            [Category("Scripting"), Description("Executes a script.")]
            public string ExecuteScript(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string scriptName = string.Empty;

                // try to retrieve the script name
                if (args != null && args.Length > 0 && args[0] != null)
                {
                    scriptName = args[0].Trim();
                }

                // check if script name is valid and present
                if (!mobjScripts.ContainsKey(scriptName))
                {
                    output.Add(Resources.InvalidScriptNameSpecified);
                }

                bool runAsync = false;
                // check if the second argument is "async", or "true" and if so the user wants to run the script asyncronesly
                if (args != null && args.Length > 1 && args[1] != null &&
                   (args[1].Trim().ToLower() == bool.TrueString.ToLower() ||  args[1].ToLower() == Resources.async.ToLower()))
                {
                    runAsync = true;
                }

                // if no errors add the script
                if (output.Count == 0)
                {
                    mobjScripts[scriptName].Execute(this.mobjManager, runAsync);
                }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? scriptName : string.Empty;
            }

            [Category("Scripting"), Description("Removes a script.")]
            public string RemoveScript(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string scriptName = string.Empty;
                try
                {
                    scriptName = string.Join(string.Empty, args);
                }
                catch { }

                if (!mobjScripts.ContainsKey(scriptName))
                {
                    output.Add(Resources.InvalidScriptNameSpecified);
                }

                // if no errors add the script
                if (output.Count == 0) { mobjScripts.Remove(scriptName); }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? scriptName : string.Empty;
            }

            [Category("Scripting"), Description("Loads a script from a file.")]
            public string LoadScript(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string scriptName = string.Empty;
                Script script = null;

                // check for valid arguments
                if (args != null && args.Length > 1 && args[0] != null)
                {
                    // check if script with that name already exists
                    if (!mobjScripts.ContainsKey(args[0].Trim()))
                    {
                        output.Add(Resources.ScriptWithThatNameExists.Replace(Resources.RKEY, args[0].Trim()));
                    }
                    else
                    {
                        // store the script name
                        scriptName = args[0].Trim();

                        // check for valid script name
                        if (!Helpers.IsValidName(scriptName))
                        {
                            output.Add(Resources.InvalidScriptNameSpecified);
                        }
                        else
                        {

                            // get file name for script file
                            string scriptFile = string.Empty;
                            try
                            {
                                scriptFile = string.Join(string.Empty, args, 1, args.Length - 1);
                            }
                            catch { }

                            // check if script file exists or not
                            if (!System.IO.File.Exists(scriptFile))
                            {
                                output.Add(Resources.FileNotFound);
                            }
                            else
                            {
                                // create a new script
                                script = new Script();

                                // everything seems ok so we load script
                                System.IO.StreamReader reader = null;
                                try
                                {
                                    reader = new System.IO.StreamReader(scriptFile);
                                    while (reader.Peek() != -1)
                                    {
                                        string Tmp = string.Empty;
                                        Tmp = reader.ReadLine();
                                        // do not add comments (standard vb style or C style language comments) or blank lines
                                        if (Tmp.Trim() != string.Empty &&
                                            !Tmp.Trim().StartsWith("\'") &&
                                            !Tmp.Trim().ToLower().StartsWith("rem") &&
                                            !Tmp.Trim().ToLower().StartsWith("//"))
                                        {
                                            // add line to script file
                                            script.mobjLines.Add(Tmp);
                                        }
                                    }
                                }
                                catch
                                {
                                    output.Add(Resources.ErrorReadingFile);
                                }
                                finally
                                {
                                    if (reader != null) reader.Close();
                                    reader = null;
                                }
                            }
                        }
                    }
                }
                else
                {
                    output.Add(Resources.InvalidScriptNameSpecified);
                }

                // if no errors add the script
                if (output.Count == 0) { mobjScripts.Add(scriptName, script); }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? scriptName : string.Empty;
            }
            #endregion

            public override void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
            {
                base.OnConnect(Manager, ExtenderManager);
                DoAddCommand(Resources.CMD_loadscript, new xGameConsoleXNA.CommandCallback(LoadScript));
                DoAddCommand(Resources.CMD_removescript, new xGameConsoleXNA.CommandCallback(RemoveScript));
                DoAddCommand(Resources.CMD_listscripts, new xGameConsoleXNA.CommandCallback(ListScripts));
                DoAddCommand(Resources.CMD_executescript, new xGameConsoleXNA.CommandCallback(ExecuteScript));
                //DoAddCommand(Resources.CMD_beginloop, new xGameConsoleXNA.CommandCallback(BeginLoop));
                //DoAddCommand(Resources.CMD_endloop, new xGameConsoleXNA.CommandCallback(EndLoop));
                //DoAddCommand(Resources.CMD_exitloop, new xGameConsoleXNA.CommandCallback(ExitLoop));
            }
        }
    }
}
