/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;
using System.ComponentModel;
using xGameConsoleXNA.Properties;


namespace xGameConsoleXNA
{
    namespace GenericCommands
    {

        public class VaribleCommands : BaseCommands
        {

            // Implements DXTools.IConsoleExtender, idisposable

            private Dictionary<string, string> mobjVaribles = new Dictionary<string, string>();

            #region " Commands ... "
            // add varible will be added at a later time
            //[Category("Varibles"), Description("Adds two numeric varibles together.")]
            //public string AddVar(string[] args, bool usedAsFunction)
            //{
            //    if (args != null)
            //    {
            //        if (mobjVaribles.ContainsKey(args[0]) == false)
            //        {
            //            if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
            //            {
            //                this.Print("A varible with the name " + args[0] + " does not exist!");
            //            }
            //            return null;
            //        }
            //        if (Helpers.IsNumeric(mobjVaribles[args[0]]) == false)
            //        {
            //            if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
            //            {
            //                this.Print("The varible does not appear to be numeric.");
            //            }
            //            return null;
            //        }
            //        if (Helpers.IsNumeric(args[1]) == false)
            //        {
            //            if (args[args.Length - 1].Trim().ToLower() != "/n" || args[args.Length - 1].Trim().ToLower() != "-n")
            //            {
            //                this.Print("A non numeric value was specified. The value was " + args[1]);
            //            }
            //            return null;
            //        }

            //        // add values together
            //        float Val1;
            //        float Val2;
            //        Val1 = System.Convert.ToSingle(mobjVaribles[args[0]]);
            //        Val2 = System.Convert.ToSingle(args[1]);
            //        mobjVaribles[args[0]] = (Val1 + Val2).ToString();
            //        // return new varible value
            //        return mobjVaribles[args[0]];
            //    }
            //}

            [Category("Varibles"), Description("Appends data to the value of a varible.")]
            public string VarAppend(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                // check if a valid varible names were specified
                if (args != null && args.Length > 0 && mobjVaribles.ContainsKey(args[0].Trim()))
                {
                    // validate second varible name
                    if (args.Length > 1 && mobjVaribles.ContainsKey(args[1].Trim()))
                    {
                        // valid names 
                    }
                    else
                    {
                        output.Add(Resources.InvalidVaribleName + " " + Resources.Second + Resources.argumentDOT);
                    }
                }
                else
                {
                    output.Add(Resources.InvalidVaribleName + " " + Resources.First + Resources.argumentDOT);
                }

                // if no erors do append
                if (output.Count == 0) mobjVaribles[args[0].Trim()] += mobjVaribles[args[1].Trim()];

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction && output.Count == 0 ? mobjVaribles[args[0].Trim()] : string.Empty;
            }

            [Category("Varibles"), Description("Creates a varible. \nExample: createvar foo 12")]
            public string CreateVar(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string value = string.Empty;
                string varName = string.Empty;

                // check if a valid varible name was specified
                if (args != null && args.Length > 0 && Helpers.IsValidName(args[0].Trim()))
                {
                    varName = args[0].Trim();
                    // valid name so check if a value was specified
                    try
                    {
                        if (args.Length > 1) value = string.Join(" ", args, 1, args.Length - 1);
                    }
                    catch { }
                }
                else
                {
                    output.Add(Resources.InvalidVaribleName + " " + Resources.InvalidCharacters);
                }

                // if no errors were outputted we can add the varible
                if (output.Count == 0) mobjVaribles.Add(varName, value);

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? varName : string.Empty;
            }

            [Category("Varibles"), Description("Sets the value of a varible.")]
            public string SetVar(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string value = string.Empty;

                if (args != null && args.Length > 0 && mobjVaribles.ContainsKey(args[0].Trim()))
                {
                    // valid name so get value to be set
                    try
                    {
                        if (args.Length > 1) value = string.Join(" ", args, 1, args.Length - 1);
                    }
                    catch { }
                }
                else
                {
                    // bad variblename specified
                    output.Add(Resources.InvalidVaribleName);
                }

                // if no output errors we can set the value
                if (output.Count == 0) mobjVaribles[args[0].Trim()] = value;

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? value : string.Empty;
            }

            [Category("Varibles"), Description("Gets the value of a varible.")]
            public string GetVar(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string value = string.Empty;

                if (args != null && args.Length > 0 && mobjVaribles.ContainsKey(args[0].Trim()))
                {
                    // valid name so get value to be set
                    value = mobjVaribles[args[0].Trim()];

                }
                else
                {
                    // bad variblename specified
                    output.Add(Resources.InvalidVaribleName);
                }

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? value : string.Empty;
            }

            [Category("Varibles"), Description("Deletes a varible.")]
            public string DeleteVar(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();
                string varName = string.Empty;

                if (args != null && args.Length > 0 && mobjVaribles.ContainsKey(args[0].Trim()))
                {
                    // valid name so get value to be set
                    varName = args[0].Trim();
                }
                else
                {
                    // bad variblename specified
                    output.Add(Resources.InvalidVaribleName);
                }

                // if no errors remove varible
                if (output.Count == 0) mobjVaribles.Remove(varName);

                if (!usedAsFunction) this.Print(output);
                return usedAsFunction ? varName : string.Empty;
            }

            [Category("Varibles"), Description("Displays a list of all the created varibles.")]
            private string ListVaribles(string[] args, bool usedAsFunction)
            {
                StringCollection output = new StringCollection();


                // add header
                output.Add(Resources.Varible + new string(' ', 20) + Resources.Value);
                output.Add(new string('-', output[output.Count - 1].Length));

                // only add vars if there are some to add
                if (mobjVaribles.Count > 0)
                {
                    // get var array
                    string[] Varibles;
                    Varibles = new string[mobjVaribles.Keys.Count];
                    mobjVaribles.Keys.CopyTo(Varibles, 0);

                    // add each varible
                    foreach (string tmp in Varibles)
                    {
                        output.Add(tmp.PadRight(Math.Max(30, tmp.Length) - Math.Min(30, tmp.Length), ' ') + mobjVaribles[tmp]);
                    }
                }

                // add number of varibles
                output.Add(mobjVaribles.Count.ToString() + " " + Resources.Varibles);

                if (!usedAsFunction) this.Print(output);
                // remove header and footer entries
                output.RemoveAt(0);
                output.RemoveAt(0);
                output.RemoveAt(output.Count - 1);

                return usedAsFunction ? Helpers.StringCollectionToString(output, "\n") : string.Empty;
            }


            #endregion

            public VaribleCommands(xGameConsoleXNA.Console Console)
                : base(Console)
            {
            }  

            public override void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager)
            {
                base.OnConnect(Manager, ExtenderManager);
                DoAddCommand(Resources.CMD_createvar, new xGameConsoleXNA.CommandCallback(this.CreateVar));
                DoAddCommand(Resources.CMD_setvar, new xGameConsoleXNA.CommandCallback(this.SetVar));
                DoAddCommand(Resources.CMD_getvar, new xGameConsoleXNA.CommandCallback(this.GetVar));
                DoAddCommand(Resources.CMD_deletevar, new xGameConsoleXNA.CommandCallback(this.DeleteVar));
                DoAddCommand(Resources.CMD_listvars, new xGameConsoleXNA.CommandCallback(this.ListVaribles));
                DoAddCommand(Resources.CMD_appendvar, new xGameConsoleXNA.CommandCallback(this.VarAppend));
                //DoAddCommand(Resources.CMD_addvar, new xGameConsoleXNA.CommandCallback(this.AddVar));
            }
        }
    }
}
