/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
// Special thanks to Jason Olson from which this code is originally based on.
// http://geekswithblogs.net/jolson/
// http://geekswithblogs.net/jolson/archive/2004/02/16/2174.aspx
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;


namespace xGameConsoleXNA
{
    /// <summary>
    /// Stores a queued list of commands.
    /// </summary>
    public class CommandBuffer : IDisposable
    {        
        // Attributes/Properties
        private Queue<string> mobjLines = new Queue<string>();

        /// <summary>
        /// Returns a reference to the lines in the command queue.
        /// </summary>
        public Queue<string> Lines
        {
            get { return mobjLines; }
        }

        /// <summary>
        /// CommandBuffer constructor.
        /// </summary>
        public CommandBuffer()
        {
        }

        /// <summary>
        /// Adds a new command to the buffer.
        /// </summary>
        /// <param name="command">The command to be executed.</param>
        public void AddLine(string command)
        {
            mobjLines.Enqueue(command);
        }

        /// <summary>
        /// Adds an array of commands to the buffer.
        /// </summary>
        /// <param name="commands">The array of commands to be executed.</param>
        public void AddLine(string[] commands)
        {
            foreach (string Tmp in commands)
            {
                mobjLines.Enqueue(Tmp);
            }
        }

        /// <summary>
        /// Adds an collection of commands to the buffer.
        /// </summary>
        /// <param name="commands">The collection of commands to be executed.</param>
        public void AddLine(StringCollection commands)
        {
            foreach (string Tmp in commands)
            {
                mobjLines.Enqueue(Tmp);
            }
        }

        #region " IDisposable Support "
        private bool disposedValue = false; // To detect redundant calls

        // IDisposable
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    // free unmanaged resources when explicitly called
                    mobjLines.Clear();
                    mobjLines = null;
                }

                // free shared unmanaged resources
            }
            this.disposedValue = true;
        }

        // This code added by Visual Basic to correctly implement the disposable pattern.
        public void Dispose()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        #endregion

    }
}
