/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
// Special thanks to Jason Olson from which this code is originally based on.
// http://geekswithblogs.net/jolson/
// http://geekswithblogs.net/jolson/archive/2004/02/16/2174.aspx
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System;
using Microsoft.Xna.Framework;

namespace xGameConsoleXNA
{

    /// <summary>
    /// Privides Console based functionality.
    /// </summary>
    public class Console : IDisposable
    {
        #region Events

        public delegate void BeforeProcessEntryLineEventHandler(object Sender, ProcessEntryLineEventArgs e);
        private BeforeProcessEntryLineEventHandler BeforeProcessEntryLineEvent;

        public event BeforeProcessEntryLineEventHandler BeforeProcessEntryLine
        {
            add
            {
                BeforeProcessEntryLineEvent = (BeforeProcessEntryLineEventHandler)System.Delegate.Combine(BeforeProcessEntryLineEvent, value);
            }
            remove
            {
                BeforeProcessEntryLineEvent = (BeforeProcessEntryLineEventHandler)System.Delegate.Remove(BeforeProcessEntryLineEvent, value);
            }
        }

        public delegate void AfterProcessEntryLineEventHandler(object Sender, PrintEventArgs e);
        private AfterProcessEntryLineEventHandler AfterProcessEntryLineEvent;

        public event AfterProcessEntryLineEventHandler AfterProcessEntryLine
        {
            add
            {
                AfterProcessEntryLineEvent = (AfterProcessEntryLineEventHandler)System.Delegate.Combine(AfterProcessEntryLineEvent, value);
            }
            remove
            {
                AfterProcessEntryLineEvent = (AfterProcessEntryLineEventHandler)System.Delegate.Remove(AfterProcessEntryLineEvent, value);
            }
        }

        public delegate void BeforePrintEventHandler(object Sender, PrintEventArgs e);
        private BeforePrintEventHandler BeforePrintEvent;

        public event BeforePrintEventHandler BeforePrint
        {
            add
            {
                BeforePrintEvent = (BeforePrintEventHandler)System.Delegate.Combine(BeforePrintEvent, value);
            }
            remove
            {
                BeforePrintEvent = (BeforePrintEventHandler)System.Delegate.Remove(BeforePrintEvent, value);
            }
        }

        public delegate void AfterPrintEventHandler(object Sender, PrintEventArgs e);
        private AfterPrintEventHandler AfterPrintEvent;

        public event AfterPrintEventHandler AfterPrint
        {
            add
            {
                AfterPrintEvent = (AfterPrintEventHandler)System.Delegate.Combine(AfterPrintEvent, value);
            }
            remove
            {
                AfterPrintEvent = (AfterPrintEventHandler)System.Delegate.Remove(AfterPrintEvent, value);
            }
        }

        #endregion

        private int mintAskingQuestion = 0;
        private string mstrLastResponceToQuestion = string.Empty;
        private CommandCallback mobjUserResponce;
        private CommandManager mobjCommandManager;
        private string mstrCancelString = Guid.NewGuid().ToString();
        private StringCollection mobjHistory = new StringCollection();
        private int mintMaxHistory = 1000;
        private System.Text.StringBuilder mobjEntryLine = new System.Text.StringBuilder();


        /// <summary>
        /// Prompts the user to provide an answer to a question.
        /// </summary>
        /// <param name="Question">The question to be asked.</param>
        /// <param name="Responce">A delegate that will hander the users responce.</param>
        public void AskQuestion(string Question, CommandCallback Responce)
        {
            this.Print(Question);
            AskQuestion(Responce);
        }

        /// <summary>
        /// Prompts the user to provide an answer to a question.
        /// </summary>
        /// <param name="Responce">A <see cref="CommandCallback"/> delegate that will handle the users responce.</param>
        public void AskQuestion(CommandCallback Responce)
        {
            if (Responce == null) return;

            mintAskingQuestion++;
            mobjUserResponce = null;
            mobjUserResponce = Responce;
        }

        private void DoCancelQuestion(bool DoCancel)
        {
            if (mintAskingQuestion == 0) return;

            mintAskingQuestion = Math.Max(mintAskingQuestion - 1, 0);
            mobjEntryLine.Remove(0, mobjEntryLine.Length);

            if (DoCancel)
            {
                mobjUserResponce(new string[] { mstrCancelString }, false); // report back with cancel responce
            }
            if (mintAskingQuestion == 0) mobjUserResponce = null;
        }

        /// <summary>
        /// Cancels the current question.
        /// </summary>
        public void CancelQuestion()
        {
            DoCancelQuestion(true);
        }

        /// <summary>
        /// Returns true if the console is waiting for a responce from the user.
        /// </summary>
        public bool IsAskingQuestion
        {
            get { return mintAskingQuestion != 0; }
        }

        /// <summary>
        /// Gets/Sets the text for the entry line.
        /// </summary>
        /// <remarks>The entry line is the line where the user types in the commands they wish to execute on the console.</remarks>
        public string Entryline
        {
            get { return mobjEntryLine.ToString(); }
            set
            {
                mobjEntryLine.Remove(0, mobjEntryLine.Length);
                mobjEntryLine.Append(value);
            }
        }

        /// <summary>
        /// Gets a collection of entries entered into the console.
        /// </summary>
        /// <remarks>History stores all the text that was written out to the console.</remarks>
        public StringCollection History
        {
            get { return mobjHistory; }
        }

        /// <summary>
        /// Gets/Sets the maximum number of entries that havve been written out to the console that the console will keep track of.
        /// </summary>
        /// <remarks>The History property stores all the text that was written out to the console.</remarks>
        public int MaxHistory
        {
            get { return mintMaxHistory; }
            set
            {
                mintMaxHistory = value;
                if (mintMaxHistory < 1) mintMaxHistory = 1;
            }
        }

        /// <summary>
        /// Console constructor.
        /// </summary>
        public Console()
        {
        }

        /// <summary>
        /// Console constructor.
        /// </summary>
        /// <param name="CommandManager">A reference to a CommandManager object.</param>
        public Console(CommandManager CommandManager)
        {
            this.mobjCommandManager = CommandManager;
        }

        /// <summary>
        /// Prints an array of text out to the console.
        /// </summary>
        /// <param name="text">The text to be printed in the console.</param>
        public void Print(string[] text)
        {
            foreach (string Tmp in text)
            {
                this.Print(Tmp);
            }
        }

        /// <summary>
        /// Prints a collection of strings out to the console.
        /// </summary>
        /// <param name="text">The collection of text to be printed in the console.</param>
        public void Print(StringCollection text)
        {
            if (text == null) return;

            foreach (string Tmp in text)
            {
                this.Print(Tmp);
            }
        }

        /// <summary>
        /// Prints text out to the console.
        /// </summary>
        /// <param name="text">The text to be printed.</param>
        public void Print(string text)
        {
            // check if printing multiple lines
            int pos = text.IndexOf("\n");
            if (pos == -1)
            {
                this.DoPrint(text);
                text = string.Empty;
            }
            else
            {
                while (pos != -1)
                {
                    this.DoPrint(text.Substring(0, pos));
                    text = text.Substring(pos + 1);
                    pos = text.IndexOf("\n");
                }

            }
            if (text != string.Empty) this.DoPrint(text);
        }

        private void DoPrint(string Text)
        {
            PrintEventArgs peargs = this.RaiseBeforePrintEvent(Text);
            if (peargs.Cancel == false)
            {
                mobjHistory.Add(peargs.Text);
                this.RaiseAfterPrintEvent(peargs.Text);
            }

            // cleanup exess entries
            while (mobjHistory.Count > mintMaxHistory)
            {
                mobjHistory.RemoveAt(mintMaxHistory - 1); // -1 because of zero-based index
            }
        }

        private PrintEventArgs RaiseBeforePrintEvent(string Text)
        {
            PrintEventArgs e = new PrintEventArgs(false, Text);
            if (BeforePrintEvent != null)
                BeforePrintEvent(this, e);
            return e;
        }

        private PrintEventArgs RaiseAfterPrintEvent(string Text)
        {
            PrintEventArgs e = new PrintEventArgs(false, Text);
            if (AfterPrintEvent != null)
                AfterPrintEvent(this, e);
            return e;
        }

        /// <summary>
        /// Prints a blank line out to the console.
        /// </summary>
        public void PrintBlankLine()
        {
            this.DoPrint(string.Empty);
        }

        /// <summary>
        /// Prints a string to the entry line.
        /// </summary>
        /// <param name="Index">The index of the insertion point where the string will be inserted.</param>
        /// <param name="text">The text to be entered.</param>
        public void PrintStringToEntryLine(int Index, string text)
        {
            mobjEntryLine.Insert(Index, text);
        }
        
        /// <summary>
        /// Prints a string to the entry line.
        /// </summary>
        /// <param name="Index">The index of the insertion point where the string will be inserted.</param>
        /// <param name="text">The text to be entered.</param>
        /// <param name="ProcessEntry">If true the entry line will be processed.</param>
        /// <param name="Execute">If true the entry line will be executed.</param>
        public void PrintStringToEntryLine(int Index, string text, bool ProcessEntry, bool Execute)
        {
            mobjEntryLine.Insert(Index, text);
            if (ProcessEntry) this.ProcessEntryLine(Execute);
        }

        /// <summary>
        /// Prints a string to the entry line.
        /// </summary>
        /// <param name="text">The text to be entered.</param>
        /// <param name="ProcessEntry">If true the entry line will be processed.</param>
        /// <param name="Execute">If true the entry line will be executed.</param>
        public void PrintStringToEntryLine(string text, bool ProcessEntry, bool Execute)
        {
            mobjEntryLine.Append(text);
            if (ProcessEntry) this.ProcessEntryLine(Execute);
        }

        /// <summary>
        /// Equivalent to pressing the backspace key on the keyboard.
        /// </summary>
        /// <param name="Index">The index from which the backspacing will occour.</param>
        /// <param name="Count">The number of times backspacing will occour.</param>
        public void Backspace(int Index, int Count)
        {
            if (Index < 0) return;
            for (int idx = 0; idx <= Count - 1; idx++)
            {
                if (mobjEntryLine.Length == 0) break;
                if (Index > mobjEntryLine.Length - 1) break;
                mobjEntryLine.Remove(Index, 1);
            }
        }

        /// <summary>
        /// Equivalent to pressing the backspace key on the keyboard.
        /// </summary>
        /// <param name="Count">The number of times backspacing will occour.</param>
        public void Backspace(int Count)
        {
            for (int idx = 0; idx <= Count - 1; idx++)
            {
                if (mobjEntryLine.Length == 0) break;
                mobjEntryLine.Remove(mobjEntryLine.Length - 1, 1);
            }
        }

        /// <summary>
        /// Processes the entry line.
        /// </summary>
        /// <param name="Execute">If true the entry line will be executed.</param>
        /// <remarks>If the console is asking a question the text in the entry line will become the responce to the question. 
        /// Otherwise the entry line will be added to the command manager command buffer of the execute parameter is false.</remarks>
        public void ProcessEntryLine(bool Execute)
        {
            string text = mobjEntryLine.ToString();
            if (text.Length == 0) return;

            // check if asking question
            if (mintAskingQuestion != 0)
            {
                // store the responce
                mstrLastResponceToQuestion = text;
                // call the delegate that handles the question
                if (mobjUserResponce != null)
                {
                    mobjUserResponce.Invoke(new string[] { text }, false);
                }
                DoCancelQuestion(false);
            }
            else
            {
                if (mobjCommandManager != null)
                {
                    // clear the entry line
                    mobjEntryLine.Remove(0, mobjEntryLine.Length);
                    // check if executing
                    if (Execute)
                    {
                        mobjCommandManager.Execute(text);
                    }
                    else
                    {
                        // if not executing then add to command manager buffer
                        mobjCommandManager.Buffer.AddLine(text);
                    }
                }
            }
        }

        /// <summary>
        /// Gets/Sets a reference to a command manager.
        /// </summary>
        public CommandManager CommandManager
        {
            get { return mobjCommandManager; }
            set { mobjCommandManager = value; }
        }

        #region " IDisposable Support "
        private bool disposed = false;
        // IDisposable
        private void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    mintAskingQuestion = 0;
                    mobjUserResponce = null;
                    mobjCommandManager = null;
                }
                //  put code to free unmanaged resources here
            }
            this.disposed = true;
        }

        // This code added by Visual Basic to correctly implement the disposable pattern.
        public virtual void Dispose()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~Console()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(false);
            //base.Finalize();
        }
        #endregion

        /// <summary>
        /// Returns the last responce that was used to answer the previous question.
        /// </summary>
        public string LastResponceToQuestion
        {
            get { return mstrLastResponceToQuestion; }
        }

        /// <summary>
        /// Gets/Sets the value of the cancel string.
        /// </summary>
        /// <remarks>The cancel string is a special string that is used to inform the console to cancel out of the current question it may be asking.</remarks>
        public string CancelString
        {
            get { return mstrCancelString; }
            set { mstrCancelString = value; }
        }

    }

}
