/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;


namespace xGameConsoleXNA
{
    /// <summary>
    /// Provides a interface for connecting and disconnecting command extender objects to the command manager.
    /// </summary>
    public interface ICommandExtender
    {
        /// <summary>
        /// Should return true if OnConnect was successfull.
        /// </summary>
        bool IsConnected { get; }
        /// <summary>
        /// Connects the object to the extender manager.
        /// </summary>
        /// <param name="Manager">A reference to a CommandManager object.</param>
        /// <param name="ExtenderManager">A reference to a CommandExtenderManager object.</param>
        /// <remarks>Place code in this method for adding commands to the command manager. Commands that are added
        /// to the command manager should be removed with a call to the OnDisconnect method.</remarks>
        void OnConnect(CommandManager Manager, CommandExtenderManager ExtenderManager);
        /// <summary>
        /// Disconnects the object from the command manager.
        /// </summary>
        /// <remarks>The object should remove any commands from the extender manager if commands were previously added
        /// during the call to the OnConnect method.</remarks>
        void OnDisconnect();
    }

}
