/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace xGameConsoleXNA
{
    /// <summary>
    /// Provides a delegate for rendering the console.
    /// </summary>
    /// <param name="gameTime">Time passed since the last call to Microsoft.Xna.Framework.Game.Update.</param>
    /// <example>A simplistic example of how to use the <see cref="Console.RenderConsoleMethod"/> property.
    /// <code>
    /// xGameConsoleXNA.Console console;
    /// console.RenderConsoleMethod = this.Render();
    /// 
    /// void Render(GameTime gameTime) {
    ///     // Place render code here
    /// }
    /// </code>
    /// </example>
    public delegate void RenderConsole(GameTime gameTime);
   /// <summary>
    /// Provides a delegate that is called when a command is executed.
    /// </summary>
    /// <param name="args">The array of arguments to be passed to the command.</param>
    /// <param name="usedAsFunction">If set to true informs the command that it is being used like a function.</param>
    /// <returns>Returns the result that the command produced.</returns>
    public delegate string CommandCallback(string[] args, bool usedAsFunction);
    /// <summary>
    /// Provides a delegate that returns information about a command and it's return value.
    /// </summary>
    /// <param name="valueReturned">The value that the command returned.</param>
    /// <param name="Command">The command that was executed.</param>
    /// <param name="index">The index of the command that was executed.</param>
    public delegate void ReturnValueCallback(string valueReturned, string Command, int index);
       /// <summary>
    /// A callback delegate used by the CommandExtenderManager.LoadExtenderObjects method.
    /// </summary>
    /// <param name="e">The exection that was raised.</param>
    public delegate void LoadExtenderException(Exception e);
    /// <summary>
    /// A callback delegate used by the CommandExtenderManager.LoadExtenderObjects method.
    /// </summary>
    /// <param name="e">A reference to an object that was sucsessfully loaded.</param>
    public delegate void ICommandExtenderLoaded(ICommandExtender e);

}
