/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System.Diagnostics;
//using System.Data;
using System.Collections;
using System.Collections.Generic;
using System;

namespace xGameConsoleXNA
{
    /// <summary>
    /// A class for handling return value events.
    /// </summary>
    public class ReturnValueEventArgs : System.ComponentModel.CancelEventArgs
    {
        /// <summary>
        /// The command that was called.
        /// </summary>
        protected string mstrCommand = string.Empty;
        /// <summary>
        /// The arguments that were used.
        /// </summary>
        protected string[] mstrArgs = null;
        /// <summary>
        /// The return value that the command returned.
        /// </summary>
        protected string mstrReturnValue = string.Empty;
        /// <summary>
        /// The full command and arguments that aws called as it was typed into the entry line.
        /// </summary>
        protected string commandString;


        /// <summary>
        /// Gets only the arguments that were specified for the command.
        /// </summary>
        public string ArgumentString
        {
            get { return commandString.Substring(this.mstrCommand.Length + 1); }
        }

        /// <summary>
        /// The full command and arguments that was called as it was typed into the entry line.
        /// </summary>
        public string CommandString
        {
            get { return commandString; }
        }

        /// <summary>
        /// The name of the command that was called.
        /// </summary>
        public string Command
        {
            get { return mstrCommand; }
        }

        /// <summary>
        /// The arguments that were passed to the comamnd.
        /// </summary>
        public string[] Args
        {
            get { return mstrArgs; }
        }

        /// <summary>
        ///  The return value that the command retuned.
        /// </summary>
        public string ReturnValue
        {
            get { return mstrReturnValue; }
        }

        /// <summary>
        ///  ReturnValueEventArgs commands.
        /// </summary>
        public ReturnValueEventArgs()
        {
        }

        /// <summary>
        ///  ReturnValueEventArgs commands.
        /// </summary>
        /// <param name="Command">The name of the command that was called.</param>
        /// <param name="Args">The arguments that were passed to the comamnd.</param>
        /// <param name="ReturnValue">The return value that the command retuned.</param>
        public ReturnValueEventArgs(string Command, string[] Args, string ReturnValue)
        {
            this.mstrCommand = Command;
            this.mstrArgs = Args;
            this.mstrReturnValue = ReturnValue;
        }

        /// <summary>
        ///  ReturnValueEventArgs commands.
        /// </summary>
        /// <param name="Command">The name of the command that was called.</param>
        /// <param name="Args">The arguments that were passed to the comamnd.</param>
        public ReturnValueEventArgs(string Command, string[] Args)
        {
            this.mstrCommand = Command;
            this.mstrArgs = Args;
        }

        /// <summary>
        ///  ReturnValueEventArgs commands.
        /// </summary>
        /// <param name="Command">The name of the command that was called.</param>
        /// <param name="Args">The arguments that were passed to the comamnd.</param>
        /// <param name="ReturnValue">The return value that the command retuned.</param>
        /// <param name="Cancel">Sets the initial value of the Cancel property.</param>
        public ReturnValueEventArgs(string Command, string[] Args, string ReturnValue, bool Cancel)
        {
            this.mstrCommand = Command;
            this.mstrArgs = Args;
            this.mstrReturnValue = ReturnValue;
            this.Cancel = Cancel;
        }

        /// <summary>
        ///  ReturnValueEventArgs commands.
        /// </summary>
        /// <param name="Command">The name of the command that was called.</param>
        /// <param name="Args">The arguments that were passed to the comamnd.</param>
        /// <param name="Cancel">Sets the initial value of the Cancel property.</param>
        public ReturnValueEventArgs(string Command, string[] Args, bool Cancel)
        {
            this.mstrCommand = Command;
            this.mstrArgs = Args;
            this.Cancel = Cancel;
        }

        /// <summary>
        ///  ReturnValueEventArgs commands.
        /// </summary>
        /// <param name="cmdStr">The full command and arguments that was called as it was typed into the entry line.</param>
        /// <param name="Command">The name of the command that was called.</param>
        /// <param name="Args">The arguments that were passed to the comamnd.</param>
        /// <param name="ReturnValue">The return value that the command retuned.</param>
        public ReturnValueEventArgs(string cmdStr, string Command, string[] Args, string ReturnValue)
            : this(Command, Args, ReturnValue)
        {
            this.commandString = cmdStr;
        }

        /// <summary>
        ///  ReturnValueEventArgs commands.
        /// </summary>
        /// <param name="cmdStr">The full command and arguments that was called as it was typed into the entry line.</param>
        /// <param name="Command">The name of the command that was called.</param>
        /// <param name="Args">The arguments that were passed to the comamnd.</param>
        public ReturnValueEventArgs(string cmdStr, string Command, string[] Args)
            : this(Command, Args)
        {
            this.commandString = cmdStr;
        }

        /// <summary>
        ///  ReturnValueEventArgs commands.
        /// </summary>
        /// <param name="cmdStr">The full command and arguments that was called as it was typed into the entry line.</param>
        /// <param name="Command">The name of the command that was called.</param>
        /// <param name="Args">The arguments that were passed to the comamnd.</param>
        /// <param name="ReturnValue">The return value that the command retuned.</param>
        /// <param name="Cancel">Sets the initial value of the Cancel property.</param>
        public ReturnValueEventArgs(string cmdStr, string Command, string[] Args, string ReturnValue, bool Cancel)
            : this(Command, Args, ReturnValue, Cancel)
        {
            this.commandString = cmdStr;
        }

        /// <summary>
        ///  ReturnValueEventArgs commands.
        /// </summary>
        /// <param name="cmdStr">The full command and arguments that was called as it was typed into the entry line.</param>
        /// <param name="Command">The name of the command that was called.</param>
        /// <param name="Args">The arguments that were passed to the comamnd.</param>
        /// <param name="Cancel">Sets the initial value of the Cancel property.</param>
        public ReturnValueEventArgs(string cmdStr, string Command, string[] Args, bool Cancel)
            : this(Command, Args, Cancel)
        {
            this.commandString = cmdStr;
        }
    }
}
