/*
Created by: X
http://www.createdbyx.com/

xGameConsoleXNA is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
#define Win32

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace xGameConsoleXNA
{
    /// <summary>
    /// Provides an interface for accessing Keyboard mouse and gamepad info.
    /// </summary>
    public interface IUserInputService
    {
        /// <summary>
        /// The current state of the gamepad's A button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's A button.</returns>
        ButtonState A(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's B button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's B button.</returns>
        ButtonState B(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's X button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's X button.</returns>
        ButtonState X(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's Y button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's Y button.</returns>
        ButtonState Y(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's Back button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's Back button.</returns>
        ButtonState Back(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's Start button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's Start button.</returns>
        ButtonState Start(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's left sholder button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's left joystick button.</returns>
        ButtonState LeftSholder(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's left joystick button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's left joystick button.</returns>
        ButtonState LeftStick(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's right sholder button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's right sholder button.</returns>
        ButtonState RightSholder(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's right joystick button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's right joystick button.</returns>
        ButtonState RightStick(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's left DPad button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's left DPad button.</returns>
        ButtonState Left(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's right DPad button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's right DPad button.</returns>
        ButtonState Right(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's up DPad button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's up DPad button.</returns>
        ButtonState Up(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's down DPad button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's down DPad button.</returns>
        ButtonState Down(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's left thumb stick.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's left thumb stick.</returns>
        Vector2 LeftThumbStick(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's right thumb stick.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's right thumb stick.</returns>
        Vector2 RightThumbStick(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's left trigger.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's left trigger.</returns>
        float LeftTrigger(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's right trigger.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the current state of the gamepad's right trigger.</returns>
        float RightTrigger(PlayerIndex index);
        /// <summary>
        /// The current state of the gamepad's connection status.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the game pad is connected.</returns>
        bool IsConnected(PlayerIndex index);
        /// <summary>
        /// Determin if the gamepad's left thumb stick has changed.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the state of the gamepad's left thumb stick changed.</returns>
        bool LeftThumbStickChanged(PlayerIndex index);
        /// <summary>
        /// Determin if the gamepad's right thumb stick has changed.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the state of the gamepad's right thumb stick changed.</returns>
        bool RightThumbStickChanged(PlayerIndex index);
        /// <summary>
        /// Determin if the state of the gamepad's left trigger has changed.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the gamepad's left trigger was changed.</returns>
        bool LeftTriggerChanged(PlayerIndex index);
        /// <summary>
        /// Determin if the state of the gamepad's right trigger has changed.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the gamepad's right trigger was changed.</returns>
        bool RightTriggerChanged(PlayerIndex index);
        /// <summary>
        /// Gets the state of a keyboard key.
        /// </summary>
        /// <param name="key">The key to check.</param>
        /// <returns>Will return true if the key is pressed.</returns>
        bool IsKeyDown(Keys key);
        /// <summary>
        /// Gets the state of a keyboard key.
        /// </summary>
        /// <param name="key">The key to check.</param>
        /// <returns>Will return true if the key is released.</returns>
        bool IsKeyUp(Keys key);
        /// <summary>
        /// Gets an array of keys that are being pressed on the keyboard.
        /// </summary>
        /// <returns>Returns an array of pressed keys on the keyboard.</returns>
        Keys[] PressedKeys();
        /// <summary>
        /// Gets an array of keys that are not being pressed on the keyboard.
        /// </summary>
        /// <returns>Returns an array of released keys on the keyboard.</returns>
        Keys[] ReleasedKeys();
        /// <summary>
        /// Gets an array of keys that were previously in a pressed state.
        /// </summary>
        /// <returns>Returns an array of previously pressed keys on the keyboard.</returns>
        Keys[] PreviouslyPressedKeys();
        /// <summary>
        /// Gets an array of keys that were previously in a released state.
        /// </summary>
        /// <returns>Returns an array of previously released keys on the keyboard.</returns>
        Keys[] PreviouslyReleasedKeys();

#if Win32
        /// <summary>
        /// Gets/Sets the current state of the left mouse button.
        /// </summary>
        bool IsLeftMouseButtonDown { get; set; }
        /// <summary>
        /// Gets/Sets the current state of the right mouse button.
        /// </summary>
        bool IsRightMouseButtonDown { get;  set;}
        /// <summary>
        /// Gets/Sets the current state of the middle mouse button.
        /// </summary>
        bool IsMiddleMouseButtonDown { get; set; }
        /// <summary>
        /// Gets/Sets the current state of the XButton1 mouse button.
        /// </summary>
        bool IsXButton1MouseButtonDown { get; set; }
        /// <summary>
        /// Gets/Sets the current state of the XButton2 mouse button.
        /// </summary>
        bool IsXButton2MouseButtonDown { get; set; }
        /// <summary>
        /// Gets/Sets the current state of the left mouse button.
        /// </summary>
        bool IsLeftMouseButtonUp { get;  set;}
        /// <summary>
        /// Gets/Sets the current state of the right mouse button.
        /// </summary>
        bool IsRightMouseButtonUp { get;  set;}
        /// <summary>
        /// Gets/Sets the current state of the middle mouse button.
        /// </summary>
        bool IsMiddleMouseButtonUp { get;  set;}
        /// <summary>
        /// Gets/Sets the current state of the XButton1 mouse button.
        /// </summary>
        bool IsXButton1MouseButtonUp { get; set; }
        /// <summary>
        /// Gets/Sets the current state of the XButton2 mouse button.
        /// </summary>
        bool IsXButton2MouseButtonUp { get;  set;}
        /// <summary>
        /// Gets/Sets the current x location of the mouse.
        /// </summary>
        int MouseX { get; set; }
        /// <summary>
        /// Gets/Sets the current y location of the mouse.
        /// </summary>
        int MouseY { get; set; }
        /// <summary>
        /// Gets/Sets the current position of the mouse.
        /// </summary>
        Vector2 MousePosition { get; set; }
        /// <summary>
        /// Gets/Sets the current scroll wheel value of the mouse.
        /// </summary>
        int MouseScrollWheel { get; set; }
#endif

        /// <summary>
        /// The previous state of the gamepad's A button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's A button.</returns>
        ButtonState PreviousA(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's B button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's B button.</returns>
        ButtonState PreviousB(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's X button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's X button.</returns>
        ButtonState PreviousX(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's Y button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's Y button.</returns>
        ButtonState PreviousY(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's Back button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's Back button.</returns>
        ButtonState PreviousBack(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's Start button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's Start button.</returns>
        ButtonState PreviousStart(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's left sholder button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's left joystick button.</returns>
        ButtonState PreviousLeftSholder(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's left joystick button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's left joystick button.</returns>
        ButtonState PreviousLeftStick(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's right sholder button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's right sholder button.</returns>
        ButtonState PreviousRightSholder(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's right joystick button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's right joystick button.</returns>
        ButtonState PreviousRightStick(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's left DPad button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's left DPad button.</returns>
        ButtonState PreviousLeft(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's right DPad button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's right DPad button.</returns>
        ButtonState PreviousRight(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's up DPad button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's up DPad button.</returns>
        ButtonState PreviousUp(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's down DPad button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's down DPad button.</returns>
        ButtonState PreviousDown(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's left thumb stick.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's left thumb stick.</returns>
        Vector2 PreviousLeftThumbStick(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's right thumb stick.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's right thumb stick.</returns>
        Vector2 PreviousRightThumbStick(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's left trigger.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's left trigger.</returns>
        float PreviousLeftTrigger(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's right trigger.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns the previous state of the gamepad's right trigger.</returns>
        float PreviousRightTrigger(PlayerIndex index);
        /// <summary>
        /// The previous state of the gamepad's connection status.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the game pad was connected.</returns>
        bool WasConnected(PlayerIndex index);
        /// <summary>
        /// Gets the previous state of a keyboard key.
        /// </summary>
        /// <param name="key">The key to check.</param>
        /// <returns>Will return true if the key was pressed.</returns>
        bool WasKeyDown(Keys key);
        /// <summary>
        /// Gets the previous state of a keyboard key.
        /// </summary>
        /// <param name="key">The key to check.</param>
        /// <returns>Will return true if the key was released.</returns>
        bool WasKeyUp(Keys key);


#if Win32
        /// <summary>
        /// Gets/Sets the previous state of the left mouse button.
        /// </summary>
        bool WasLeftMouseButtonDown { get; set; }
        /// <summary>
        /// Gets/Sets the previous state of the right mouse button.
        /// </summary>
        bool WasRightMouseButtonDown { get;  set;}
        /// <summary>
        /// Gets/Sets the previous state of the middle mouse button.
        /// </summary>
        bool WasMiddleMouseButtonDown { get; set; }
        /// <summary>
        /// Gets/Sets the previous state of the XButton1 mouse button.
        /// </summary>
        bool WasXButton1MouseButtonDown { get; set; }
        /// <summary>
        /// Gets/Sets the previous state of the XButton2 mouse button.
        /// </summary>
        bool WasXButton2MouseButtonDown { get; set; }
        /// <summary>
        /// Gets/Sets the previous state of the left mouse button.
        /// </summary>
        bool WasLeftMouseButtonUp { get;  set;}
        /// <summary>
        /// Gets/Sets the previous state of the right mouse button.
        /// </summary>
        bool WasRightMouseButtonUp { get;  set;}
        /// <summary>
        /// Gets/Sets the previous state of the middle mouse button.
        /// </summary>
        bool WasMiddleMouseButtonUp { get;  set;}
        /// <summary>
        /// Gets/Sets the previous state of the XButton1 mouse button.
        /// </summary>
        bool WasXButton1MouseButtonUp { get; set; }
        /// <summary>
        /// Gets/Sets the previous state of the XButton2 mouse button.
        /// </summary>
        bool WasXButton2MouseButtonUp { get;  set;}
        /// <summary>
        /// Gets/Sets the previous x location of the mouse.
        /// </summary>
        int PreviousMouseX { get; set; }
        /// <summary>
        /// Gets/Sets the previous y location of the mouse.
        /// </summary>
        int PreviousMouseY { get; set; }
        /// <summary>
        /// Gets/Sets the previous position of the mouse.
        /// </summary>
        Vector2 PreviousMousePosition { get; set; }
        /// <summary>
        /// Gets/Sets the previous scroll wheel value of the mouse.
        /// </summary>
        int PreviousMouseScrollWheel { get; set; }
#endif


        /// <summary>
        /// Determins if the A button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool AClick(PlayerIndex index);
        /// <summary>
        /// Determins if the B button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool BClick(PlayerIndex index);
        /// <summary>
        /// Determins if the X button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool XClick(PlayerIndex index);
        /// <summary>
        /// Determins if the Y button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool YClick(PlayerIndex index);
        /// <summary>
        /// Determins if the Back button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool BackClick(PlayerIndex index);
        /// <summary>
        /// Determins if the Start button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool StartClick(PlayerIndex index);
        /// <summary>
        /// Determins if the left sholder button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool LeftSholderClick(PlayerIndex index);
        /// <summary>
        /// Determins if the left stick button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool LeftStickClick(PlayerIndex index);
        /// <summary>
        /// Determins if the right sholder button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool RightSholderClick(PlayerIndex index);
        /// <summary>
        /// Determins if the right stick button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool RightStickClick(PlayerIndex index);
        /// <summary>
        /// Determins if the left DPad button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool LeftClick(PlayerIndex index);
        /// <summary>
        /// Determins if the right DPad button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool RightClick(PlayerIndex index);
        /// <summary>
        /// Determins if the up DPad button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool UpClick(PlayerIndex index);
        /// <summary>
        /// Determins if the down DPad button was clicked.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool DownClick(PlayerIndex index);
        /// <summary>
        /// Determins if a keyboard key was clicked.
        /// </summary>
        /// <param name="key">The keyboard key to check.</param>
        /// <returns>Returns true if the key was pressed then released.</returns>
        bool KeyClick(Keys key);
        /// <summary>
        /// Gets an array of clicked keyboard keys.
        /// </summary>
        /// <returns>Returns the keys that were pressed then released.</returns>
        Keys[] GetClickedKeys();


        /// <summary>
        /// Sets the status for the A button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void A(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the B button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void B(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the X button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void X(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the Y button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void Y(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the back button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void Back(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the start button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void Start(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the left holder button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void LeftSholder(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the left joystick button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void LeftStick(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the right sholder button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void RightSholder(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the right joystick button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void RightStick(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the DPad left button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void Left(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the DPad right button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void Right(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the DPad up button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void Up(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the DPad down button.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the button as.</param>
        void Down(PlayerIndex index, ButtonState state);
        /// <summary>
        /// Sets the status for the left thumbstick.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the thumb stick as.</param>
        void LeftThumbStick(PlayerIndex index, Vector2 value);
        /// <summary>
        /// Sets the status for the right thumbstick.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the thumb stick as.</param>
        void RightThumbStick(PlayerIndex index, Vector2 value);
        /// <summary>
        /// Sets the status for the left trigger.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the trigger as.</param>
        void LeftTrigger(PlayerIndex index, float value);
        /// <summary>
        /// Sets the status for the right trigger.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the trigger as.</param>
        void RightTrigger(PlayerIndex index, float value);
        /// <summary>
        /// Sets the connection status for the gamepad.
        /// </summary>
        /// <param name="index">The index of the player.</param>
        /// <param name="state">The specified state to set the connection as.</param>
        void IsConnected(PlayerIndex index, bool value);
        /// <summary>
        /// Gets the current state of a keyboard key.
        /// </summary>
        /// <param name="key">The key to be checked.</param>
        /// <returns>Returns Down if the key is down or Up if the key is released.</returns>
        KeyState GetKeyState(Keys key);
        /// <summary>
        /// Sets the current state of a keyboard key.
        /// </summary>
        /// <param name="key">The key to be set.</param>
        /// <param name="state">The state to set the key as.</param>
        void SetKeyState(Keys key, KeyState state);
        /// <summary>
        /// Gets the previous state of a keyboard key.
        /// </summary>
        /// <param name="key">The key to be checked.</param>
        /// <returns>Returns Down if the key is down or Up if the key is released.</returns>
        KeyState GetPreviuosKeyState(Keys key);
        /// <summary>
        /// Sets the previous state of a keyboard key.
        /// </summary>
        /// <param name="key">The key to be set.</param>
        /// <param name="state">The state to set the key as.</param>
        void SetPreviuosKeyState(Keys key, KeyState state);

#if Win32
        /// <summary>
        /// Determins if the left mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool LeftMouseButtonClicked { get; }
        /// <summary>
        /// Determins if the right mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool RightMouseButtonClicked { get; }
        /// <summary>
        /// Determins if the middle mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool MiddleMouseButtonClicked { get; }
        /// <summary>
        /// Determins if the XButton1 mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool XButton1MouseButtonClicked { get; }
        /// <summary>
        /// Determins if the Xbutton2 mouse button button was clicked.
        /// </summary>
        /// <returns>Returns true if the button was pressed then released.</returns>
        bool XButton2MouseButtonClicked { get; }
        /// <summary>
        /// Determins if the X position of the mouse has changed.
        /// </summary>
        /// <returns>Returns true if the mouse has changed position.</returns>
        bool MouseXChanged { get; }
        /// <summary>
        /// Determins if the Y position of the mouse has changed.
        /// </summary>
        /// <returns>Returns true if the mouse has changed position.</returns>
        bool MouseYChanged { get; }
        /// <summary>
        /// Determins if the scroll wheel position of the mouse has changed.
        /// </summary>
        /// <returns>Returns true if the mouse scroll wheel has changed position.</returns>
        bool MouseScrollWheelChanged { get; }
#endif

        /// <summary>
        /// Updates the input state of the keyboard, mouse and gamepad's.
        /// </summary>
        /// <param name="gameTime">Time elapsed since the last call to Update(Microsoft.Xna.Framework.GameTime)</param>
        void Update(GameTime gameTime);
    }
}
