/*
Code written/modified by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
  
 Original source for this code comes from http://www.ziggyware.com/
 Unfortunatly I could not determin the exact name of the original author to give him/her credit.
 */

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using System.Xml;
using xGameConsoleXNA.Properties;
#endregion

namespace xGameConsoleXNA
{
    public class XNAFont : IDisposable
    {
        public struct BatchItem
        {
            public int X, Y;
            public Color Color;
            public string Text;

            public BatchItem(string txt, int x, int y, Color c)
            {
                this.X = x;
                this.Y = y;
                this.Text = txt;
                this.Color = c;
            }
        }

        struct XNAFontUV
        {
            public float u, v;
            public int Width;

            public XNAFontUV(float u, float v, int Width)
            {
                this.u = u;
                this.v = v;
                this.Width = Width;
            }
        }

        string strFontFamily;
        Texture2D texture;
        SpriteBatch fontRenderer;
        public int Height;
        public int LineSpacing = 0;
        public int Kerning = 0;
        Dictionary<string, XNAFontUV> UVData = new Dictionary<string, XNAFontUV>();

        public Texture2D Texture
        {
            get { return texture; }
            set { texture = value; }
        }

        public void DrawText(IList<BatchItem> items)
        {
            foreach (BatchItem b in items)
            {
                this.DrawText(b.Text, b.X, b.Y, b.Color);
            }
        }

        public void DrawText(string str, int x, int y, Color col)
        {
            int iCurrentX = x;
            int iCurrentY = y;

            fontRenderer.Begin(SpriteBlendMode.AlphaBlend);
            for (int idx = 0; idx <= str.Length - 1; idx++)
            {
                string c;
                c = str[idx].ToString();
                XNAFontUV uv = UVData[c];

                // check for line break
                if (c == @"\" && idx < str.Length - 1 && str[idx + 1].ToString() == "n")
                {
                    iCurrentX = x;
                    iCurrentY += Height + LineSpacing;
                }
                else // regular processing
                {
                    if (c != " ")
                    {
                        fontRenderer.Draw(texture,
                            new Rectangle(iCurrentX, iCurrentY,
                                uv.Width, Height),
                            new Rectangle((int)uv.v,
                                          (int)uv.u,
                                          uv.Width, Height),
                            col);
                    }
                }

                iCurrentX += uv.Width - Kerning;
            }
            
            fontRenderer.End();
        }

        public static XNAFont LoadFont(GraphicsDevice device, string strXML, string strDDS)
        {
            System.IO.Stream s = System.IO.File.OpenRead(strXML);
            return LoadFont(device, s, strDDS);
        }

        public static XNAFont LoadFont(GraphicsDevice device, System.IO.Stream strXML, string strDDS)
        {
            XNAFont ret = new XNAFont();

            System.Xml.XmlTextReader tr = new XmlTextReader(strXML);
            while (tr.Read())
            {
                if (tr.Name == Resources.XML_font)
                {
                    if (tr.AttributeCount > 0)
                    {
                        ret.strFontFamily = tr.GetAttribute(Resources.XML_family);
                        ret.Height = Convert.ToInt32(tr.GetAttribute(Resources.XML_height));

                    }
                }
                else if (tr.Name == Resources.XML_char)
                {
                    string str = tr.GetAttribute(Resources.XML_c);
                    str = str.Replace("&qwo;", "\"");
                    str = str.Replace("&amp;", "&");
                    str = str.Replace("&lt;", "<");
                    str = str.Replace("&gt;", ">");
                    char c = str[0];

                    float u, v;
                    int w;
                    if (float.TryParse(tr.GetAttribute(Resources.XML_u), out u) &&
                        float.TryParse(tr.GetAttribute(Resources.XML_v), out v) &&
                        int.TryParse(tr.GetAttribute(Resources.XML_width), out w))
                    {
                        // all data was retrieved ok so add character entry
                        ret.UVData.Add(c.ToString(), new XNAFontUV(u, v, w));
                    }
                    else
                    {
                        throw new System.IO.InvalidDataException(Resources.CouldNotDeciferCharacterEntry.Replace(Resources.RKEY, str));
                    }
                }
            }
            tr.Close();

            if (strDDS != string.Empty) ret.texture = Texture2D.FromFile(device, strDDS);
            ret.fontRenderer = new SpriteBatch(device);

            return ret;
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (texture != null)
                texture.Dispose();

            if (fontRenderer != null)
                fontRenderer.Dispose();

            texture = null;
            fontRenderer = null;
        }

        #endregion
    }
}
