/*
Created by: X
http://www.createdbyx.com/
*/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace xGameConsoleXNA_Example2
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        xGameConsoleXNA.ConsoleComponent console;
        KeyboardState prevstate;


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
            // set the window text to let the user know how to bring down the console
            this.Window.Title = "Example 2: Press Escape to show/hide the console";
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            this.IsMouseVisible = true;
            this.Window.AllowUserResizing = true;

            // create console object
            console = new xGameConsoleXNA.ConsoleComponent(this);

            // add the custom commands object and specify true to have the extender manager 
            // connect it
            this.console.ExtenderManager.Add(new CustomCommands(this, this.console.Console));

            // add prebuilt generic console commands to give the user some basic common console functionality
            this.console.ExtenderManager.Add(new xGameConsoleXNA.GenericCommands.GenericCommands(this.console.Console));

            // write a message out to the console letting people know how to get started
            this.console.Print("Type 'commands' and press enter for a list of availible commands ...");

            // now add the console to the components list
            this.Components.Add(console);

            base.Initialize();

        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // set the font for the console
               // this.console.Font = content.Load<SpriteFont>("CourierNew");
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // check if user is opening console using the escape key
            KeyboardState state = Keyboard.GetState();
            if (state.IsKeyUp(Keys.Escape) && prevstate.IsKeyDown(Keys.Escape))
            {
                this.console.Show();
            }

            // store state
            this.prevstate = state;

            // position the console
            MouseState mouse = Mouse.GetState();
            if (mouse.LeftButton == ButtonState.Pressed)
            {
                this.console.Left = mouse.X;
                this.console.Top = mouse.Y;
            }
            if (mouse.RightButton == ButtonState.Pressed)
            {
                this.console.Width = mouse.X - this.console.Left;
                this.console.Height = mouse.Y - this.console.Top;
            }

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // No draw code needed because the console component will use it's default internal rendering system
            // that can be replaced if desired

            base.Draw(gameTime);
        }
    }
}